from typing import Optional, DefaultDict, List, Dict, Callable
from collections import defaultdict
from pathlib import Path
import os, time

from .core import get_files, tail, read, parse_line, emit_all

class GSCClient:
    def __init__(
        self, 
        filepath: Optional[str] = os.path.join(os.environ["LOCALAPPDATA"], "Plutonium", "storage", "t6", "raw", "scriptdata")
    ) -> None:
        self.filepath = filepath
        self._events: DefaultDict[str, List] = defaultdict(list)
        self._positions: Dict[Path, int] = {}

    def clear_events(self) -> None:
        for log in os.listdir(self.filepath):
            if log.startswith("event_") and log.endswith(".jsonl"):
                path = os.path.join(self.filepath, log)
                os.remove(path)

    def on(self, event: str) -> Callable:
        def decorator(func) -> Callable:
            self._events[event].append(func)
            return func
        return decorator

    def run(self) -> None:
        self._positions = {file: tail(file) for file in get_files(Path(self.filepath))}
        try:
            while True:
                files = list(get_files(Path(self.filepath)))
                results = {
                    file: read(file, self._positions.get(file, 0)) for file in files
                }

                events = filter(None, (
                    parse_line(line) for lines, _ in results.values() for line in lines
                ))

                emit_all(self._events, events)
                self._positions = {
                    file: offset for file, (_, offset) in results.items()
                }
                time.sleep(1)

        except KeyboardInterrupt:
            exit()


from setuptools import setup, find_packages

setup(
    name="gsc_events",
    version="0.1.4",
    author="budiworld",
    author_email="budi.world@yahoo.com",
    description="Python library for capturing and handling game events in Plutonium T6",
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    url="https://github.com/Yallamaztar/gsc-events", 
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.13',
)



from collections.abc import Iterable
from typing import Dict, Tuple, List
from pathlib import Path

def get_files(path: Path) -> Iterable[Path]:
    return filter(Path.is_file, path.glob("event_*.jsonl"))

def tail(files: Iterable[Path]) -> Dict[Path, int]:
    return {file: file.stat().st_size for file in files}

def read(file: Path, offset: int) -> Tuple[List[str], str]:
    with open(file, "r", encoding="utf-8", errors="ignore") as f:
        f.seek(offset)
        lines = f.read().splitlines()
        return lines, f.tell()


from typing import Optional, Tuple, List, Dict, Callable
from collections.abc import Iterable
import json

def parse_line(line: str) -> Optional[Tuple[str, List]]:
    try:
        data = json.loads(line)
        event = data.get("event")
        args  = data.get("args", [])
        if not event:
            return
        return event, args
    
    except json.JSONDecodeError:
        print("[gsc-events] json decode error")
        return
    
def emit_all(
    handlers: Dict[str, List[Callable]],
    events: Iterable[Tuple[str, List]]
) -> None:
    for event, args in events:
        for func in handlers.get(event, []):
            try:
                func(*args)
            except Exception as e:
                print(f"[gsc-events] Error in handler '{func.__name__}': {e}")